@@ -37,6 +37,7 @@
 
 #include <string>
 #include <sstream>
+#include "literal.h"
 #include "lexical_cast.h"
 #include "predicate.h"
 
@@ -46,8 +47,9 @@ namespace clx {
 		//  is_reserved
 		/* ----------------------------------------------------------------- */
 		template <class CharT>
-		bool is_reserved(CharT c) {
-			if (is_alpha()(c) || is_any_of("!#$&'()*+,-./:;=?@_~")(c)) return true;
+		inline bool is_reserved(CharT c) {
+			std::basic_string<CharT> ctrls(LITERAL("!#$&'()*+,-./:;=?@_~"));
+			if (is_alnum()(c) || is_any_of(ctrls.c_str())(c)) return true;
 			return false;
 		}
 	};
@@ -57,12 +59,16 @@ namespace clx {
 		//  encode
 		/* ----------------------------------------------------------------- */
 		template <class Ch, class Tr>
-		std::basic_string<Ch, Tr> encode(const std::basic_string<Ch, Tr>& s) {
+		inline std::basic_string<Ch, Tr> encode(const std::basic_string<Ch, Tr>& s) {
+			Ch ws = LITERAL(' ');
+			Ch plus = LITERAL('+');
+			Ch percent = LITERAL('%');
+			
 			std::basic_ostringstream<Ch, Tr> ss;
-			for (unsigned int i = 0; i < s.size(); i++) {
-				if (s.at(i) == ' ') ss << '+';
+			for (unsigned int i = 0; i < s.size(); ++i) {
+				if (s.at(i) == ws) ss << plus;
 				else if (!clx::detail::is_reserved(s.at(i))) {
-					ss << '%' << std::hex << static_cast<unsigned int>(s.at(i));
+					ss << percent << std::hex << static_cast<unsigned int>(s.at(i));
 				}
 				else ss << s.at(i);
 			}
@@ -73,7 +79,7 @@ namespace clx {
 		//  encode
 		/* ----------------------------------------------------------------- */
 		template <class CharT>
-		std::basic_string<CharT> encode(const CharT* s) {
+		inline std::basic_string<CharT> encode(const CharT* s) {
 			std::basic_string<CharT> tmp(s);
 			return encode(tmp);
 		}
@@ -82,10 +88,12 @@ namespace clx {
 		//  decode
 		/* ----------------------------------------------------------------- */
 		template <class Ch, class Tr>
-		std::basic_string<Ch, Tr> decode(const std::basic_string<Ch, Tr>& s) {
+		inline std::basic_string<Ch, Tr> decode(const std::basic_string<Ch, Tr>& s) {
+			Ch percent = LITERAL('%');
+			
 			std::basic_ostringstream<Ch, Tr> ss;
-			for (unsigned int i = 0; i < s.size(); i++) {
-				if (s.at(i) == '%') {
+			for (unsigned int i = 0; i < s.size(); ++i) {
+				if (s.at(i) == percent) {
 					unsigned int c = lexical_cast<unsigned int>(s.substr(i + 1, 2), std::ios::hex);
 					ss << static_cast<Ch>(c);
 					i += 2;
@@ -99,7 +107,7 @@ namespace clx {
 		//  decode
 		/* ----------------------------------------------------------------- */
 		template <class CharT>
-		std::basic_string<CharT> decode(const CharT* s) {
+		inline std::basic_string<CharT> decode(const CharT* s) {
 			std::basic_string<CharT> tmp(s);
 			return decode(tmp);
 		}
